package cqcli::importquery;

use strict;
use warnings;

require Exporter;

use Error qw(:try);
use cqcli::Exception;
use Inline::Java qw(caught cast) ;
use cqcli::Util;
use cqcli::IOHandler;
use cqcli::commonservice;
use Text::ParseWords;
use Getopt::Long 2.34;
use Encode;


our @ISA = qw(Exporter);

our %EXPORT_TAGS = ( 'all' => [ qw() ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(importquery);

our $VERSION = '0.01';

#######################################################################
our @ARGVBAACKUP = @ARGV;
our %RepetitionCount = ();
our $isLoginOpt = undef;
our $CliSession = undef;
our $JavaReqObj = undef;
our %CmdInfo = ( 'cmdstring' => [ ], 'cmdname'  => "import_query");
our %CmdLogin = ('database'	=>	undef,'schema'=>  undef,
				 'user'	=>	undef,'password'=>  undef);
our $BLANK_MARKER_SET = 0;

our $QueryName = undef;
our $FileName = undef;
our $Force = 0;
our $type = "Defect";



#########################################################################

#########handle getopt warnings begin########################

sub InitParser
{
	my @tmpARGV = @_;
	@ARGVBAACKUP = @ARGV;
	@ARGV = undef;
	$isLoginOpt = undef;
	%RepetitionCount = ();
	%CmdInfo = ('cmdstring' => [ ], 'cmdname'  => "import_query");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);

	$QueryName = undef;
	$FileName = undef;
	$Force = 0;
	
	$CmdInfo{'cmdstring'} =[@tmpARGV];
	($CmdInfo{'cmdname'}  , @ARGV ) = @{$CmdInfo{'cmdstring'}};
	$CliSession = GetSession();
	$JavaReqObj = GetJavaHash();
	return 1;
}

sub CleanUpParser()
{

	@ARGV = @ARGVBAACKUP;
	%RepetitionCount = ();
	%CmdInfo = ('cmdstring' => [ ], 'cmdname'  => "import_query");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);

	$QueryName = undef;
	$FileName = undef;
	$Force = 0;
	
	$CmdInfo{'cmdstring'} = undef;
	$CmdInfo{'cmdname'} =undef;
	$CliSession = undef;
	$JavaReqObj = undef;

}

#########handle getopt warnings end#####################################

sub handle_help_usage
{
	my $opt = $_[0];
	
	if($opt eq "help")	{
		ThrowUsageException($CmdInfo{'cmdname'},$opt);
	}
	else{
		ThrowUsageException($CmdInfo{'cmdname'},$opt);
	}
	return 1;
}

sub handle_ask_mode
{
	my $mode = shift;
	SetCQCLIAskMode(1);
	return 1;
}


sub handle_login_data
{
	my ($opt , $val) = @_;
	my $msg = undef;
	$isLoginOpt = 1;

	if($RepetitionCount{$opt}) {
		ThrowSyntaxErrorException("CQCLI_Repeat_Option","-$opt", $CmdInfo{'cmdname'} );
	}else {
		$RepetitionCount{$opt} = 1;
	}

	if( ($_[0] eq "database") && ($val)	){
		$CmdLogin{'database'} = $val;
	}elsif(($_[0] eq "user") && ($val)) {
		$CmdLogin{'user'} = $val;
	}elsif(($_[0] eq "password") && ($val)) {
		$CmdLogin{'password'} = $val;
	}elsif(($_[0] eq "password") && ($val eq "")){
		$CmdLogin{'password'} = "";
		$BLANK_MARKER_SET = 1 ;
	}
	elsif((($_[0] eq "schema") || ($_[0] eq "master")) && ($val) ) {
		$CmdLogin{'schema'} = $val; 
	}else{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt",$CmdInfo{'cmdname'});
	}
	return 1;
}

sub handle_query_option
{
	my ($opt , $name) = @_;
	my $msg = undef;
	if($RepetitionCount{'$opt'} ) {
		ThrowSyntaxErrorException("CQCLI_Repeat_Option","-$opt", $CmdInfo{'cmdname'} );
	}else {
		$RepetitionCount{'$opt'} = 1;
	}

	if($name){
		$QueryName = $name;
	}else {
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt", $CmdInfo{'cmdname'} );
	}
	return 1;
}

sub handle_recordtype_list
{
	
	my ($opt , $name) = @_;
	
	if($RepetitionCount{'$opt'} ) {
		ThrowSyntaxErrorException("CQCLI_Repeat_Option","-$opt", $CmdInfo{'cmdname'} );
	}else {
		$RepetitionCount{'$opt'} = 1;
	}
	
	$type = "Defect";
	return 1;
}

sub handle_force_option
{
	$Force = 1;
}


sub parseForHelpUsage()
{
	my $ret = undef;

	my $helpRules = {'help|h|?'		=>	\&handle_help_usage	,
					 'man'			=>	\&handle_help_usage	};
	
	Getopt::Long::Configure('autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$helpRules} );
	Getopt::Long::Configure('default');

	return 1;
}

sub parseForCoreOptions()
{
	my $ret = undef;
	my $CoreOptRules = {
			'replace'   =>	\&handle_force_option 
	};
	Getopt::Long::Configure( 'autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$CoreOptRules} );
	Getopt::Long::Configure('default');

	return 1;
}

sub parseForLoginCredentials 
{
	my $ret = undef;
	my $CommonRules = {
		'database|dbs|d:s'					=>  \&handle_login_data			,
		'user|login|u|l:s'					=>	\&handle_login_data			,
		'schema|s|m|master:s'				=>	\&handle_login_data			,
		'password|p:s'						=>	\&handle_login_data			,
		'type:s'							=>	\&handle_recordtype_list			
	};
	Getopt::Long::Configure( 'autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$CommonRules} );
	Getopt::Long::Configure('default');
	return 1;
}


sub parseForUnknownOptions
{
	my @tmpARGV = @ARGV;
	my $lmt = $#tmpARGV;
	
	if($lmt < 0) {
		ThrowSyntaxErrorException("CQCLI_Missing_Value","<file name> and <query name>", $CmdInfo{'cmdname'} );
	}
	
	if($lmt == 1) {
		for (my $cnt = 0; $cnt < $lmt;$cnt++)  
		{	
				$FileName = $tmpARGV[$cnt+1];		
				$QueryName = $tmpARGV[$cnt];	
				$cnt = $cnt + 1;
		}
	}
	else {
		ThrowSyntaxErrorException("CQCLI_Missing_Value","<query name>", $CmdInfo{'cmdname'} );
	}

	return 1;
}

sub Parse()
{
	try {

		parseForHelpUsage();
		parseForLoginCredentials();
		parseForCoreOptions();
		parseForUnknownOptions();
		
		if(!($FileName)){
			ThrowSyntaxErrorException("CQCLI_Missing_Value","file name", $CmdInfo{'cmdname'} );
		}
		elsif(!($QueryName)) {
			ThrowSyntaxErrorException("CQCLI_Missing_Value","query name", $CmdInfo{'cmdname'} );
		}
		
		if(!($FileName =~ /.qry/ )) {
			$FileName = $FileName . ".qry";
		}


		#######################################################

		if($ENV{CQCLI_LOCALE} eq 'zh') {
			$FileName = encode('cp936',$FileName);
		}
		elsif($ENV{CQCLI_LOCALE} eq 'jp') {
			$FileName = encode('cp932',$FileName);
		}
		elsif($ENV{CQCLI_LOCALE} eq 'en') {
			$FileName  = encode('cp1252',$FileName);
		}

		if(!(-e $FileName)){
			ThrowException('CQCLI_Failure_FileOpen',$FileName);
		}

		if($ENV{CQCLI_LOCALE} eq 'zh') {
			$FileName = decode('cp936',$FileName);
		}
		elsif($ENV{CQCLI_LOCALE} eq 'jp') {
			$FileName = decode('cp932',$FileName);
		}
		elsif($ENV{CQCLI_LOCALE} eq 'en') {
			$FileName  = decode('cp1252',$FileName);
		}

		########################################################

		$JavaReqObj->put("name",$QueryName);
		$JavaReqObj->put("file",$FileName);
		$JavaReqObj->put("type",$type);

		if($Force) {
			$JavaReqObj->put("replace","true");
		}
	}
	catch cqcli::Exception with {
		my $ex = shift;	
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_ErrJava_ParserUnknown");
		}
		else{
			ThrowParseException('CQCLI_Err_ParserUnknown',$@);
		}
    }
	finally {
		Getopt::Long::Configure('default');
	};

	return 1;
}

sub ValidateSessionInfo()
{
	
	if($isLoginOpt){
		ValidateLoginCredentials();
		return;
	}elsif($CliSession){
		return;
	}else{
		ValidateLoginCredentials();
		return;
	}

}
sub ValidateLoginCredentials()
{
	if(!($CmdLogin{'user'})){
		$CmdLogin{'user'} = GetUserName();
	}
	if(!($CmdLogin{'password'}) && $BLANK_MARKER_SET == 0){
		$CmdLogin{'password'} = GetPassword();
	}
	if(!($CmdLogin{'database'})){
		$CmdLogin{'database'} = GetDBName();
	}
	if(!($CmdLogin{'schema'})){
		$CmdLogin{'schema'} = GetSchemaName();
	}
	$CliSession=CreateSession(%CmdLogin);
	# is not a command scope session
	SetSessionScope(0);
}

sub Validate()
{
	try {
		ValidateSessionInfo();
	}
	catch cqcli::Exception with {
		my $ex = shift;		
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_ErrJava_ValidationUnknown");
		}
		else{
			ThrowValidationException('CQCLI_Err_ValidationUnknown',$@);
		}		
    }
	finally {	};

	return 1;
}

sub importquery
{
	my @tmpCmd = @_;
	my $req = undef;
	
	try {
		InitParser(@tmpCmd);
		Parse();
		Validate();
		$req =  GetJavaPayload("import_query",$JavaReqObj);
		SetRequestObject($req);
		return 1;


	}finally {
		CleanUpParser();
	};
}

1;